/*
   This file is part of TALER
   Copyright (C) 2022, 2026 Taler Systems SA

   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 3, or (at your option) any later version.

   TALER is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   TALER; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
/**
 * @file src/backenddb/iterate_accounts_by_instance.c
 * @brief Implementation of the iterate_accounts_by_instance function for Postgres
 * @author Christian Grothoff
 */
#include "platform.h"
#include <taler/taler_pq_lib.h>
#include "merchant-database/iterate_accounts_by_instance.h"
#include "helper.h"


/**
 * Context for iterate_accounts().
 */
struct SelectAccountsContext
{
  /**
   * Function to call with the results.
   */
  TALER_MERCHANTDB_AccountCallback cb;

  /**
   * Closure for @e cb.
   */
  void *cb_cls;

  /**
   * Database context.
   */
  struct TALER_MERCHANTDB_PostgresContext *pg;

  /**
   * Merchant instance ID.
   */
  const char *id;

  /**
   * Set to the return value on errors.
   */
  enum GNUNET_DB_QueryStatus qs;

};


/**
 * Function to be called with the results of a SELECT statement
 * that has returned @a num_results results about accounts.
 *
 * @param cls of type `struct SelectAccountsContext *`
 * @param result the postgres result
 * @param num_results the number of results in @a result
 */
static void
select_account_cb (void *cls,
                   PGresult *result,
                   unsigned int num_results)
{
  struct SelectAccountsContext *lic = cls;

  for (unsigned int i = 0; i < num_results; i++)
  {
    struct TALER_FullPayto payto;
    char *facade_url = NULL;
    char *extra_wire_subject_metadata = NULL;
    json_t *credential = NULL;
    struct TALER_MERCHANTDB_AccountDetails acc;
    struct TALER_MerchantPrivateKeyP merchant_priv;
    bool no_priv;
    struct GNUNET_PQ_ResultSpec rs[] = {
      GNUNET_PQ_result_spec_auto_from_type ("h_wire",
                                            &acc.h_wire),
      GNUNET_PQ_result_spec_auto_from_type ("salt",
                                            &acc.salt),
      GNUNET_PQ_result_spec_string ("payto_uri",
                                    &payto.full_payto),
      GNUNET_PQ_result_spec_allow_null (
        GNUNET_PQ_result_spec_string ("credit_facade_url",
                                      &facade_url),
        NULL),
      GNUNET_PQ_result_spec_allow_null (
        GNUNET_PQ_result_spec_string ("extra_wire_subject_metadata",
                                      &extra_wire_subject_metadata),
        NULL),
      GNUNET_PQ_result_spec_allow_null (
        TALER_PQ_result_spec_json ("credit_facade_credentials",
                                   &credential),
        NULL),
      GNUNET_PQ_result_spec_bool ("active",
                                  &acc.active),
      GNUNET_PQ_result_spec_allow_null (
        GNUNET_PQ_result_spec_auto_from_type ("merchant_priv",
                                              &merchant_priv),
        &no_priv),
      GNUNET_PQ_result_spec_end
    };

    if (GNUNET_OK !=
        GNUNET_PQ_extract_result (result,
                                  rs,
                                  i))
    {
      GNUNET_break (0);
      lic->qs = GNUNET_DB_STATUS_HARD_ERROR;
      return;
    }
    acc.instance_id = lic->id;
    acc.payto_uri = payto;
    acc.credit_facade_url = facade_url;
    acc.credit_facade_credentials = credential;
    acc.extra_wire_subject_metadata = extra_wire_subject_metadata;
    if (NULL != lic->cb)
      lic->cb (lic->cb_cls,
               no_priv ? NULL : &merchant_priv,
               &acc);
    GNUNET_PQ_cleanup_result (rs);
  }
}


enum GNUNET_DB_QueryStatus
TALER_MERCHANTDB_iterate_accounts_by_instance (
  struct TALER_MERCHANTDB_PostgresContext *pg,
  const char *id,
  TALER_MERCHANTDB_AccountCallback cb,
  void *cb_cls)
{
  struct SelectAccountsContext lic = {
    .cb = cb,
    .cb_cls = cb_cls,
    .pg = pg,
    .id = id
  };
  struct GNUNET_PQ_QueryParam params[] = {
    GNUNET_PQ_query_param_string (id),
    GNUNET_PQ_query_param_end
  };
  enum GNUNET_DB_QueryStatus qs;

  TMH_PQ_prepare_anon (pg,
                       "SELECT"
                       " ma.h_wire"
                       ",ma.salt"
                       ",ma.payto_uri"
                       ",ma.credit_facade_url"
                       ",ma.credit_facade_credentials::TEXT"
                       ",ma.extra_wire_subject_metadata"
                       ",ma.active"
                       ",mi.merchant_priv"
                       " FROM merchant_accounts ma"
                       " JOIN merchant.merchant_instances mi"
                       "  ON (mi.merchant_id=$1)");
  qs = GNUNET_PQ_eval_prepared_multi_select (pg->conn,
                                             "",
                                             params,
                                             &select_account_cb,
                                             &lic);
  if (0 > lic.qs)
    return lic.qs;
  return qs;
}
